@@ -43,11 +43,14 @@ __long_description__   -- triple-quoted string containing long
 __start_line__         -- contents of the line in README where the
                           long description should start (i.e.,
                           this line and all after it until the
-                          end line will be included)
+                          end line will be included), or the line
+                          number where it should start
 __end_line__           -- contents of the line in README where the
                           long description should end (i.e., this
                           line and all after it will *not* be
-                          included)
+                          included), or the line number where it
+                          should end (negative numbers count from
+                          the end of the file)
 
 __author__             -- author name
 __author_email__       -- author e-mail address
@@ -246,7 +249,7 @@ script's globals as described above.
 import sys
 import os
 
-SETUPHELPER_VERSION = (0, 4)
+SETUPHELPER_VERSION = (0, 4, 2)
 
 def version_tuple(ver_s, dev=False):
     """
@@ -439,6 +442,13 @@ if sys.version < '2.5':
     # Now monkeypatch distutils with our extended class
     distutils.dist.DistributionMetadata = DistributionMetadata
 
+# Mapping of platforms to allowed archive extensions
+
+archive_extensions = {
+    'darwin': [".dmg", ".tar.gz", ".tar", ".zip"],
+    'linux2': [".tar.gz", ".tar.bz2", ".zip", ".tar"],
+    'win32': [".exe", ".zip", ".tar.gz", ".tar"] }
+
 def distutils_default():
     """ Support function for Python-provided distutils. """
     
@@ -458,13 +468,6 @@ def distutils_default():
         user_agent = "Python-urllib/%s SetupHelper/%s" % (
             urllib2.__version__, SETUPHELPER_VERSION)
         
-        def __init__(self, requires, opts, debug=False):
-            self._requires = requires
-            self._debug = debug
-            
-            self.setup_opts = opts
-            self.srcdir = ""
-        
         # NOTE: this is *not* fully general URL detection and parsing; it only
         # does the minimum necessary to work with HTML pages like those on PyPI
         # that contain properly formatted URLs within relatively "clean" content
@@ -497,12 +500,16 @@ def distutils_default():
                     self.params, self.query, self.fragment = m.groups()        
         
         try:
-            allowed_extensions = {
-                'darwin': [".dmg", ".tar.gz", ".tar"],
-                'linux2': [".tar.gz", ".tar.bz2", ".zip", ".tar"],
-                'win32': [".exe", ".zip", ".tar.gz", ".tar"] }[sys.platform]
+            allowed_extensions = archive_extensions[sys.platform]
         except KeyError:
-            allowed_extensions = ["tar.gz", ".tar"] # reasonably safe choice
+            allowed_extensions = ["tar.gz", ".zip"] # reasonably safe choice
+        
+        def __init__(self, requires, opts, debug=False):
+            self._requires = requires
+            self._debug = debug
+            
+            self.setup_opts = opts
+            self.srcdir = ""
         
         def find_pattern(data, pattern, klass=None):
             r = re.compile(pattern)
@@ -662,9 +669,9 @@ def distutils_default():
                         opts = {'.gz': 'z', '.bz2': 'j'}[ext[4:]]
                     except KeyError:
                         opts = ""
-                    unpack_cmd = "tar -x%svf " % opts
+                    unpack_cmd = "tar -x%svf" % opts
                 elif ext == ".zip":
-                    unpack_cmd = "unzip "
+                    unpack_cmd = "unzip"
                 else:
                     unpack_cmd = ""
                 
@@ -705,20 +712,30 @@ def distutils_default():
                 os.chdir("deps")
                 
                 if not clean:
-                    # Unpack the source archive in the deps directory
-                    unpack_dir = "%s-%s" % (req, version)
-                    if os.path.isdir(unpack_dir):
-                        print unpack_dir, "was previously unpacked."
-                    else:
-                        unpack_cmdline = "%s%s" % (unpack_cmd, filename)
-                        print "Executing", unpack_cmdline
-                        os.system(unpack_cmdline)
+                    if unpack_cmd:
+                        # Unpack the source archive in the deps directory
+                        unpack_dir = "%s-%s" % (req, version)
+                        if os.path.isdir(unpack_dir):
+                            print unpack_dir, "was previously unpacked."
+                        else:
+                            unpack_cmdline = "%s %s" % (unpack_cmd, filename)
+                            print "Executing", unpack_cmdline
+                            os.system(unpack_cmdline)
+                    elif "install" in self.setup_opts:
+                        # The archive can be executed directly (exe), but we only
+                        # do this if we're installing
+                        print "Executing", filename
+                        os.system(filename)
+                        unpack_dir = None
                 else:
-                    unpack_dir = [dname for dname in os.listdir(".") if os.path.isdir(dname)
-                        and dname.startswith(req)][0]
+                    try:
+                        unpack_dir = [dname for dname in os.listdir(".") if os.path.isdir(dname)
+                            and dname.startswith(req)][0]
+                    except IndexError:
+                        unpack_dir = None
                 
                 # Chdir to the source root and run setup with same args as ours
-                if os.path.isdir(unpack_dir):
+                if unpack_dir and os.path.isdir(unpack_dir):
                     os.chdir(unpack_dir)
                     process_cmdline = "python setup.py %s" % self.setup_opts
                     print "Executing", process_cmdline
@@ -963,29 +980,49 @@ class SetupHelper(object):
     def process_readme(self):
         # crib our long description from the opening section of
         # the README file
-        if hasattr(self, '__start_line__') and hasattr(self, '__end_line__'):
+        if hasattr(self, '__start_line__') or hasattr(self, '__end_line__'):
             if not hasattr(self, '__long_description__'):
+                for attrname in ('__start_line__', '__end_line__'):
+                    if not hasattr(self, attrname):
+                        setattr(self, attrname, None)
+                
+                if isinstance(self.__start_line__, int):
+                    startline = self.__start_line__
+                elif self.__start_line__ is None:
+                    startline = 0
+                else:
+                    startline = None
+                if isinstance(self.__end_line__, int):
+                    endline = self.__end_line__
+                else:
+                    endline = None
+                
                 readme = open("README", 'rU')
-                lines = []
-                lastline = ""
-                started = False
                 try:
-                    for line in readme:
-                        line = line.strip()
-                        if started:
-                            if line == self.__end_line__:
-                                break
-                            else:
-                                lines.append(lastline)
-                        else:
-                            if line == self.__start_line__:
-                                started = True
-                        lastline = line
+                    lines = readme.readlines()
                 finally:
                     readme.close()
-                self.__long_description__ = '\n'.join(lines)
-            del self.__start_line__
-            del self.__end_line__
+                
+                if (endline is not None) and (endline < 0):
+                    endline = endline + len(lines)
+                desclines = []
+                started = False
+                for lineno, line in enumerate(lines):
+                    line = line.rstrip('\n')
+                    if started:
+                        if (lineno == endline) or (line == self.__end_line__):
+                            break
+                        else:
+                            desclines.append(line)
+                    elif (lineno == startline) or (line == self.__start_line__):
+                        desclines.append(line)
+                        started = True
+                
+                self.__long_description__ = '\n'.join(desclines)
+            
+            for attrname in ('__start_line__', '__end_line__'):
+                if hasattr(self, attrname):
+                    delattr(self, attrname)
     
     def process_vars(self):
         # We'll store the lines that need to go into MANIFEST.in here (and add
